@@ -4,7 +4,7 @@ use strict;
 use warnings;
 
 use vars qw($VERSION @ISA);
-$VERSION = '0.08';
+$VERSION = '0.09';
 
 #--------------------------------------------------------------------------
 
@@ -33,12 +33,13 @@ use base qw(WWW::Scraper::ISBN::Driver);
 # Modules
 
 use WWW::Mechanize;
+use JSON::XS;
 
 ###########################################################################
 # Constants
 
-use constant	REFERER	=> 'https://www.waterstones.com';
-use constant	SEARCH	=> 'https://www.waterstones.com/index/search?term=';
+use constant REFERER => 'https://www.waterstones.com';
+use constant SEARCH  => 'https://www.waterstones.com/index/search?term=';
 my ($URL1,$URL2) = ('http://www.waterstones.com/book/','/[^?]+\?b=\-3\&amp;t=\-26\#Bibliographicdata\-26');
 
 #--------------------------------------------------------------------------
@@ -82,10 +83,10 @@ website.
 =cut
 
 sub search {
-	my $self = shift;
-	my $isbn = shift;
-	$self->found(0);
-	$self->book(undef);
+    my $self = shift;
+    my $isbn = shift;
+    $self->found(0);
+    $self->book(undef);
 
     # validate and convert into EAN13 format
     my $ean = $self->convert_to_ean13($isbn);
@@ -93,14 +94,14 @@ sub search {
         if(!$ean || (length $isbn == 13 && $isbn ne $ean)
                  || (length $isbn == 10 && $isbn ne $self->convert_to_isbn10($ean)));
 
-	my $mech = WWW::Mechanize->new();
+    my $mech = WWW::Mechanize->new();
     $mech->agent_alias( 'Linux Mozilla' );
     $mech->add_header( 'Accept-Encoding' => undef );
     $mech->add_header( 'Referer' => REFERER );
 
     eval { $mech->get( SEARCH . $ean ) };
     return $self->handler("The Waterstones website appears to be unavailable.")
-	    if($@ || !$mech->success() || !$mech->content());
+        if($@ || !$mech->success() || !$mech->content());
 
 #print STDERR "\n# search=[".SEARCH."$ean]\n";
 #print STDERR "\n# is_html=".$mech->is_html().", content type=".$mech->content_type()."\n";
@@ -111,40 +112,52 @@ sub search {
     my $url = $response->header( 'X-Meta-Og-Url' );
 #print STDERR "\n# url=[$url]\n";
 
-	return $self->handler("Failed to find that book on the Waterstones website. [$isbn]")
-	    if($url eq REFERER || $url eq REFERER . "/books/search/term/$ean");
+    return $self->handler("Failed to find that book on the Waterstones website. [$isbn]")
+        if($url eq REFERER || $url eq REFERER . "/books/search/term/$ean");
 
     eval { $mech->get( $url ) };
-	return $self->handler("Failed to find that book on the Waterstones website. [$isbn]")
-	    if($@ || !$mech->success() || !$mech->content());
+    return $self->handler("Failed to find that book on the Waterstones website. [$isbn]")
+        if($@ || !$mech->success() || !$mech->content());
 
-	# The Book page
+    # The Book page
     my $html = $mech->content();
 
-	return $self->handler("Failed to find that book on the Waterstones website. [$isbn]")
-		if($html =~ m|<strong>Sorry!</strong> We did not find any results for|si);
+    return $self->handler("Failed to find that book on the Waterstones website. [$isbn]")
+        if($html =~ m|<strong>Sorry!</strong> We did not find any results for|si);
 
-	return $self->handler("Waterstones website has crashed. [$isbn]")
-		if($html =~ m|Exception was UseCaseError: \d+|si);
+    return $self->handler("Waterstones website has crashed. [$isbn]")
+        if($html =~ m|Exception was UseCaseError: \d+|si);
 
     $html =~ s/&amp;/&/g;
 #print STDERR "\n# content2=[\n$html\n]\n";
 
     my $data;
     ($data->{title},$data->{author})
-                                = $html =~ m!<title>(.*?)\s*by\s*(.*?) \| Waterstones.com</title>!si;
-    ($data->{binding})          = $html =~ m!<span class="book-title" itemprop="name" id="scope_book_title">.*? \((.*?)\)</span>!si;
-    ($data->{description})      = $html =~ m!<div itemprop="description" id="scope_book_description">(.*?)</div>!si;
-    ($data->{publisher})        = $html =~ m!<span itemprop="publisher">([^<]+)</span>!si;
-    ($data->{pubdate})          = $html =~ m!<meta itemprop="datePublished" content="[^"]+" />([\d\/]+)\s*</span>!si;
-    ($data->{isbn13})           = $html =~ m!<span itemprop="isbn">([^<]+)</span>!si;
-    ($data->{image})            = $html =~ m!<img itemprop="image" id="scope_book_image" src="([^"]+$ean.jpg)"!si;
+                           = $html =~ m!<title>(.*?)\s*by\s*(.*?) \| Waterstones</title>!si;
+    ($data->{binding})     = $html =~ m!<span class="book-title" itemprop="name" id="scope_book_title">.*? \((.*?)\)</span>!si;
+    ($data->{description}) = $html =~ m!<div itemprop="description" id="scope_book_description">(.*?)</div>!si;
+    ($data->{publisher})   = $html =~ m!<span itemprop="publisher">([^<]+)</span>!si;
+    ($data->{pubdate})     = $html =~ m!<meta itemprop="datePublished" content="[^"]+" />([\d\/]+)\s*</span>!si;
+    ($data->{isbn13})      = $html =~ m!<span itemprop="isbn">([^<]+)</span>!si;
+    ($data->{image})       = $html =~ m!<img itemprop="image" id="scope_book_image" src="([^"]+$ean.jpg)"!si;
+    my ($json)             = $html =~ m!<script>\s*ws_dl = \[(.*?)\]\s*</script>!si;
+#print STDERR "\n# json=[\n$json\n]\n";
+
+    if($json) {
+        $data->{json} = decode_json( $json );
+        for(qw(author title imprint publication_date format)) {
+            $data->{$_} = $data->{json}{'gtm-books'}[0]{$_};
+        }
+
+        $data->{binding} ||= $data->{format};
+        $data->{pubdate} ||= $data->{publication_date};
+    }
 
 #use Data::Dumper;
 #print STDERR "\n# data=" . Dumper($data);
 
-	return $self->handler("Could not extract data from the Waterstones result page. [$isbn]")
-		unless(defined $data);
+    return $self->handler("Could not extract data from the Waterstones result page. [$isbn]")
+        unless(defined $data);
 
     for(qw(author publisher description title)) {
         $data->{$_} =~ s/&#0?39;/'/g    if($data->{$_});
@@ -163,8 +176,8 @@ sub search {
 #use Data::Dumper;
 #print STDERR "\n# data=" . Dumper($data);
 
-	# trim top and tail
-	foreach (keys %$data) { 
+    # trim top and tail
+    foreach (keys %$data) { 
         next unless(defined $data->{$_});
         $data->{$_} =~ s!&nbsp;! !g;
         $data->{$_} =~ s/^\s+//;
@@ -174,30 +187,31 @@ sub search {
 #    my $url = $mech->uri();
 #    $url =~ s/\?.*//;
 
-	my $bk = {
-		'ean13'		    => $data->{isbn13},
-		'isbn13'		=> $data->{isbn13},
-		'isbn10'		=> $data->{isbn10},
-		'isbn'			=> $data->{isbn13},
-		'author'		=> $data->{author},
-		'title'			=> $data->{title},
-		'book_link'		=> "$url",
-		'image_link'	=> $data->{image},
-		'thumb_link'	=> $data->{thumb},
-		'description'	=> $data->{description},
-		'pubdate'		=> $data->{pubdate},
-		'publisher'		=> $data->{publisher},
-		'binding'	    => $data->{binding},
-		'pages'		    => $data->{pages},
-        'html'          => $html
-	};
+    my $bk = {
+        'ean13'       => $data->{isbn13},
+        'isbn13'      => $data->{isbn13},
+        'isbn10'      => $data->{isbn10},
+        'isbn'        => $data->{isbn13},
+        'author'      => $data->{author},
+        'title'       => $data->{title},
+        'book_link'   => "$url",
+        'image_link'  => $data->{image},
+        'thumb_link'  => $data->{thumb},
+        'description' => $data->{description},
+        'pubdate'     => $data->{pubdate},
+        'publisher'   => $data->{publisher},
+        'binding'     => $data->{binding},
+        'pages'       => $data->{pages},
+        'json'        => $data->{json},
+        'html'        => $html
+    };
 
 #use Data::Dumper;
 #print STDERR "\n# book=".Dumper($bk);
 
     $self->book($bk);
-	$self->found(1);
-	return $self->book;
+    $self->found(1);
+    return $self->book;
 }
 
 1;
@@ -208,8 +222,9 @@ __END__
 
 Requires the following modules be installed:
 
-L<WWW::Scraper::ISBN::Driver>,
-L<WWW::Mechanize>
+L<JSON::XS>,
+L<WWW::Mechanize>,
+L<WWW::Scraper::ISBN::Driver>
 
 =head1 SEE ALSO
 
@@ -236,7 +251,7 @@ be forthcoming, please feel free to (politely) remind me.
 
 =head1 COPYRIGHT & LICENSE
 
-  Copyright (C) 2010-2015 Barbie for Miss Barbell Productions
+  Copyright (C) 2010-2019 Barbie for Miss Barbell Productions
 
   This module is free software; you can redistribute it and/or
   modify it under the Artistic Licence v2.
